
import numpy as np
import pickle
from matplotlib import pyplot as plt
from matplotlib import style

################################################################################################

''' enter path to current directory'''
input_path = './'

################################################################################################

### plot top-k accuracy

data_list = ['position', 'position-height', 'position-height-distance', 'image']
acc_list = [[], [], [], []]


for i, data_type in enumerate(data_list):
    with open(input_path + f'{data_type}_top_k_acc.pkl', 'rb') as f:
        acc = pickle.load(f) ## beam prediction of model using this data_type
   
    acc_list[i] = [ele*100 for ele in acc]

    ## remove top-4 accuracy
    acc = acc_list[i][:3]
    acc.append(acc_list[i][4])

    acc_list[i] = acc

acc_list = np.array(acc_list)
acc_1 = acc_list[:, 0]
acc_2 = acc_list[:, 1]
acc_3 = acc_list[:, 2]
acc_5 = acc_list[:, 3]


## plot

style.use('dark_background')
# print(style.available)

fig, ax = plt.subplots(figsize=(10, 5))

labels = ['Position', 'Position-Height', 'Position-Height-Distance', 'Image']
x = np.arange(len(labels)) * 2  # the label locations

width = 0.22  # the width of the bars
# color = ['blue', 'green', 'red', 'purple', 'orange']

plt.ylim((0, 100))
# plt.grid(axis='y')
ax.set_axisbelow(True)
ax.yaxis.grid(color='gray')

rect = ax.bar(x - 3*width, acc_1, width, label='Top-1', color='blue')
rect = ax.bar(x - width, acc_2, width, label='Top-2', color='red')
rect = ax.bar(x + width, acc_3, width, label='Top-3', color='gray')
rect = ax.bar(x + 3*width, acc_5, width, label='Top-5', color='green')

plt.xticks(x, labels)
ax.legend(loc='lower center', bbox_to_anchor=(0.5, 1),
          fancybox=True, shadow=True, ncol=4)

fig.tight_layout()

plt.savefig(input_path + 'plots/top_k_accuracy_plot.jpg', dpi=300, bbox_inches='tight') 
# plt.show()

################################################################################################

### plot speed and height - based accuracy


data_list = ['position', 'position-height', 'position-height-distance', 'image']
hight_acc_list = [[], [], []]
speed_acc_list = [[], [], []]
mode_list = ['low', 'med', 'high']


for i, data_type in enumerate(data_list):

    with open(input_path + 'data/height_test.pkl', 'rb') as f:
        X_s_test = pickle.load(f)

    with open(input_path + 'data/speed_test.pkl', 'rb') as f:
        X_h_test = pickle.load(f)  

    with open(input_path + f'results/{data_type}_preds.pkl', 'rb') as f:
        y_pred = pickle.load(f)

    if data_type != 'image':
        path = input_path + 'data/y_true.pkl'
    else:
        path = input_path + 'data/y_true-img.pkl'

    with open(path, 'rb') as f:
        y_true = pickle.load(f)


    pair = zip(X_s_test, X_h_test, y_pred, y_true)
    arr = np.array(list(pair))


    low_height  = arr[arr[:, 1] <= 40 ]
    med_height  = arr[arr[:, 1] > 40 ]
    med_height  = med_height[med_height[:, 1] <= 80]
    high_height = arr[arr[:, 1] > 80 ]

    acc_low = accuracy_score(low_height[:, 3], low_height[:, 2])
    acc_med = accuracy_score(med_height[:, 3], med_height[:, 2])
    acc_high = accuracy_score(high_height[:, 3], high_height[:, 2])

    hight_acc_list[0].append(acc_low*100)
    hight_acc_list[1].append(acc_med*100)
    hight_acc_list[2].append(acc_high*100)


    low_speed  = arr[arr[:, 0] <= 10 ]
    med_speed  = arr[arr[:, 0] > 10 ]
    med_speed  = med_speed[med_speed[:, 0] <= 20]
    high_speed = arr[arr[:, 0] > 20 ]


    acc_low = accuracy_score(low_speed[:, 3], low_speed[:, 2])
    acc_med = accuracy_score(med_speed[:, 3], med_speed[:, 2])
    acc_high = accuracy_score(high_speed[:, 3], high_speed[:, 2])

    speed_acc_list[0].append(acc_low*100)
    speed_acc_list[1].append(acc_med*100)
    speed_acc_list[2].append(acc_high*100)


## height-based accuracy plot

hight_acc_list = np.array(hight_acc_list)
acc_low  = hight_acc_list[0]
acc_med  = hight_acc_list[1]
acc_high = hight_acc_list[2]

style.use('dark_background')

fig, ax = plt.subplots(figsize=(10, 5))

labels = ['Position', 'Position-Height', 'Position-Height-Distance', 'Image']
x = np.arange(len(labels)) * 2  # the label locations

width = 0.22  # the width of the bars
# color = ['blue', 'green', 'red', 'purple', 'orange']

plt.ylim((0, 100))
# plt.grid(axis='y')
ax.set_axisbelow(True)
ax.yaxis.grid(color='gray')

rect = ax.bar(x - 2*width, acc_low, width, label='Height <= 40 m', color='blue')
rect = ax.bar(x, acc_med, width, label='40 m < Height <= 80 m', color='red')
rect = ax.bar(x + 2*width, acc_high, width, label='Height > 80 m', color='gray')

plt.xticks(x, labels)
ax.legend(loc='lower center', bbox_to_anchor=(0.5, 1),
          fancybox=True, shadow=True, ncol=4)

fig.tight_layout()

plt.savefig(input_path + 'plots/height_accuracy_plot.jpg', dpi=300, bbox_inches='tight') 
# plt.show()

## speed-based accuracy plot

speed_acc_list = np.array(speed_acc_list)
acc_low  = speed_acc_list[0]
acc_med  = speed_acc_list[1]
acc_high = speed_acc_list[2]

style.use('dark_background')

fig, ax = plt.subplots(figsize=(10, 5))

labels = ['Position', 'Position-Height', 'Position-Height-Distance', 'Image']
x = np.arange(len(labels)) * 2  # the label locations

width = 0.22  # the width of the bars
# color = ['blue', 'green', 'red', 'purple', 'orange']

plt.ylim((0, 100))
# plt.grid(axis='y')
ax.set_axisbelow(True)
ax.yaxis.grid(color='gray')

rect = ax.bar(x - 2*width, acc_low, width, label='Speed <= 10 m', color='blue')
rect = ax.bar(x, acc_med, width, label='10 m < Speed <= 20 m', color='red')
rect = ax.bar(x + 2*width, acc_high, width, label='Speed > 20 m', color='gray')

plt.xticks(x, labels)
ax.legend(loc='lower center', bbox_to_anchor=(0.5, 1),
          fancybox=True, shadow=True, ncol=4)

fig.tight_layout()

plt.savefig(input_path + 'plots/speed_accuracy_plot.jpg', dpi=300, bbox_inches='tight') 
# plt.show()

#############################################################################################
